package eu.eudat.data.query.items.table.dmp;

import eu.eudat.data.dao.criteria.DataManagementPlanCriteria;
import eu.eudat.data.dao.criteria.DataManagementPlanPublicCriteria;
import eu.eudat.data.entities.DMP;
import eu.eudat.data.query.PaginationService;
import eu.eudat.data.query.definition.TableQuery;
import eu.eudat.queryable.QueryableList;
import eu.eudat.types.project.ProjectStateType;

import java.util.Date;
import java.util.UUID;

public class DataManagmentPlanPublicTableRequest extends TableQuery<DataManagementPlanPublicCriteria, DMP, UUID> {

    public QueryableList<DMP> applyCriteria() {
        QueryableList<DMP> query = this.getQuery();
        query.where((builder, root) -> builder.equal(root.get("isPublic"), true));
        if (this.getCriteria().getLike() != null && !this.getCriteria().getLike().isEmpty())
            query.where((builder, root) -> builder.or(
                    builder.like(builder.upper(root.get("label")), "%" + this.getCriteria().getLike().toUpperCase() + "%"),
                    builder.like(builder.upper(root.get("description")), "%" + this.getCriteria().getLike().toUpperCase() + "%")));
        if (this.getCriteria().getProjects() != null && !this.getCriteria().getProjects().isEmpty())
            query.where(((builder, root) -> root.get("project").get("id").in(this.getCriteria().getProjects())));
        if (this.getCriteria().getProjectStatus() != null) {
            if (this.getCriteria().getProjectStatus().getValue().equals(ProjectStateType.FINISHED.getValue()))
                query.where((builder, root) -> builder.lessThan(root.get("project").get("enddate"), new Date()));
            if (this.getCriteria().getProjectStatus().getValue().equals(ProjectStateType.ONGOING.getValue()))
                query.where((builder, root) ->
                        builder.or(builder.greaterThan(root.get("project").get("enddate"), new Date())
                                , builder.isNull(root.get("project").get("enddate"))));
        }
        if (this.getCriteria().datasetProfile != null && !this.getCriteria().datasetProfile.isEmpty()) query
                .where(((builder, root) -> root.get("profile").get("id").in(this.getCriteria().datasetProfile)));
        if (this.getCriteria().getDmpOrganisations() != null && !this.getCriteria().getDmpOrganisations().isEmpty()) query
                .where(((builder, root) -> root.join("organisations").get("reference").in(this.getCriteria().getDmpOrganisations())));
        return query;
    }

    @Override
    public QueryableList<DMP> applyPaging(QueryableList<DMP> items) {
        return PaginationService.applyPaging(items, this);
    }
}
